class Settings:
    """
        - Settings to use Celery and Redis
        - Define backend and broker url in celery
    """

    def __init__(self, **kwargs):
        self.backend = 'redis://localhost/0'
        self.broker  = 'redis://localhost/0'
        self.crontab = kwargs.get('crontab')

    def get_settings(self):

        print(f'\n***** Current Backend ==> {self.backend}')
        print(f'***** Current Broker ==> {self.broker}')

        if self.crontab:
            return

        print('\nIf inputs not provided, we will use the defaults ones\n')

        backend_input = input('Please your celery backend string:    ')
        broker_input  = input('Please your celery broker string:    ')

        if backend_input:
            self.backend = backend_input

        if broker_input:
            self.broker = broker_input

        return



from ..operation                            import Operation as ops


class Employee:
    _current_date = ops.get_today_date()

    def __init__(self, employee_info, excluded_employees):
        self.excluded_employees    = excluded_employees
        self.emp_id                = employee_info.get('id')
        self.name                  = employee_info.get('name')
        self.last_name             = employee_info.get('lastname')
        self.date_of_birth         = employee_info.get('dateOfBirth')
        self.emp_start_date        = employee_info.get('employmentStartDate')
        self.emp_end_date          = employee_info.get('employmentEndDate')
        self.last_notification     = employee_info.get('lastNotification')
        self.last_birth_notified   = employee_info.get('lastBirthdayNotified')
        self.dob                   = ops.str_to_date(self.date_of_birth)       if self.date_of_birth       else None
        self.__emp_start_date      = ops.str_to_date(self.emp_start_date)      if self.emp_start_date      else None
        self.__emp_end_date        = ops.str_to_date(self.emp_end_date)        if self.emp_end_date        else None
        self.__last_notification   = ops.str_to_date(self.last_notification)   if self.last_notification   else None
        self.__last_birth_notified = ops.str_to_date(self.last_birth_notified) if self.last_birth_notified else None

    @property
    def get_current_date(self):
        return self._current_date

    @property
    def is_still_working(self):
        if self.__emp_end_date:
            if self._current_date > self.__emp_end_date:
                return True
            return False
        return True

    @property
    def is_today_birthday_celebration(self):
        return (self._current_date.month == self.dob.month and self._current_date.day == self.dob.day)


    @property
    def is_today_anniversary_celebration(self):
        return (
                self._current_date.month == self.__emp_start_date.month and
                self._current_date.day == self.__emp_start_date.day
        )

    @property
    def has_started_working(self):
        return self.__emp_start_date and self._current_date >= self.__emp_start_date

    @property
    def can_receive_wishes(self):
        return self.emp_id not in self.excluded_employees

    @property
    def get_employment_dob(self):
        return self.dob

    @property
    def get_employment_start_date(self):
        return self.__emp_start_date

    @property
    def get_employment_end_date(self):
        return self.__emp_end_date

    @property
    def get_last_notification(self):
        return self.__last_notification

    @property
    def get_last_birthday_notification(self):
        return self.__last_notification



from .db.db                                      import DatabaseOperation
from .factory                                    import GetInput


def start_wish():
    """
        Run the Factory Class.
        Goes through the logic. Pass one employee at the time from the scheduler and send emails
    """

    # DB CREATION - TO HANDLE SCHEDULED TASKS
    print('\n******** PLEASE ENTER YOUR DATABASE FULLPATH *******\n')
    DB = DatabaseOperation('db.sqlite3')
    DB.create_connection()

    print("\n**************** Wishes Selections *****************\n")
    print("Please select: ** ONLY BIRTHDAY WISHES SELECTION IS ACCEPTED AT THE MOMENT")
    print("\t 1. Birthday Wishes")
    print("\t 2. Anniversary Wishes\n")

    INPUT = GetInput(input("Enter your selection:   "))
    INPUT.get_input_redirect()


if __name__ == '__main__':
    start_wish()



import pathlib
import sqlite3
import os
from sqlite3                                    import Error


class DatabaseOperation:
    """
        - Takes 1 required argument - database file name
        - Creates a DB file in the provided location or path
    """

    def __init__(self, db_file):
        self.db_file = db_file
        self.db_dir  = pathlib.Path(__file__).parents[0]

    def create_connection(self):
        """ create a database connection to a SQLite database """

        db_dir_input = input('\tDB FULLPATH:    ')
        if db_dir_input:
            self.db_dir = db_dir_input

        db_path = f'{self.db_dir}/{self.db_file}'

        if os.path.isfile(db_path):
            print('DB already exists!')
            return None

        conn = None
        try:
            conn = sqlite3.connect(db_path)
            print(f'Success ==> v{sqlite3.version}')
        except Error as e:
            print(f'Error ==> {e}')
        finally:
            if conn:
                conn.close()



class ProcessEmployee:

    def __init__(self, employee):
        self.employee = employee

    def validate_birthdays_employees(self):

        if self.employee.get_last_birthday_notification != self.employee.get_current_date:
            if self.employee.has_started_working:
                if self.employee.is_still_working:
                    if self.employee.can_receive_wishes:
                        if self.employee.is_today_birthday_celebration:
                            return True
        return False

    def validate_anniversary_employees(self):

        if self.employee.get_last_notification != self.employee.get_current_date:
            if self.employee.has_started_working:
                if self.employee.is_still_working:
                    if self.employee.can_receive_wishes:
                        if self.employee.is_today_anniversary_celebration:
                            return True
        return False



from .factory                                import GetInput


def test_birthday_wishes():

    resp = GetInput('1', crontab='Y').get_input_redirect()
    assert resp == '1'

    return resp


def test_anniversary_wishes():

    resp = GetInput('2', crontab='Y').get_input_redirect()
    assert resp == '2'

    return resp



import json
import pathlib
from threading                              import Thread


class JSONData(Thread):

    __current_dir = pathlib.Path(__file__).parent

    def __init__(self, employee, employee_not_today, call, emp_list=None, emp_na_list=None):
        super(JSONData, self).__init__()
        self.call               = call
        self.employee           = employee
        self.employee_not_today = employee_not_today
        self.employees_list     = emp_list
        self.employees_na_list  = emp_na_list
        self.db_dir             = f'{self.__current_dir}/db'

    def run(self):
        if self.employees_list and self.call == 'Y':
            self.write_wished_employees_to_json()
        if self.employees_na_list and self.call == 'N':
            self.write_not_celebrating_employees_to_json()

    @staticmethod
    def extract_employee_data(json_file):
        with open(json_file) as employees_file:
            employees = json.load(employees_file)
            return employees

    @staticmethod
    def extract_exc_employee_data():
        with open("exc_employees.json") as employees_file:
            exc_employees = json.load(employees_file)
            return exc_employees

    def write_wished_employees_to_json(self):
        with open(f'{self.__current_dir}/employees.json', 'w') as employees_file:
            json.dump(self.employees_list, employees_file)

    def write_not_celebrating_employees_to_json(self):
        with open(f'{self.__current_dir}/employees_na.json', 'w') as employees_not_today_file:
            json.dump(self.employees_na_list, employees_not_today_file)



import os
import smtplib
import ssl
import time

from dotenv                                 import load_dotenv
from threading                              import Thread


load_dotenv()

class SendWishEmail(Thread):

    def __init__(self, wish_type, employees_list, **kwargs):
        super(SendWishEmail, self).__init__()
        self.crontab         = kwargs.get('crontab')
        self.GMAIL_PORT      = os.getenv('GMAIL_PORT')
        self.GMAIL_SERVER    = os.getenv('GMAIL_SERVER')
        self.GMAIL_SENDER    = os.getenv('GMAIL_SENDER')
        self.GMAIL_PASSWORD  = os.getenv('GMAIL_PASSWORD')
        self.GMAIL_RECEIVER  = os.getenv('GMAIL_RECEIVER')
        self.birth_employees = employees_list
        self.ann_employees   = employees_list #kwargs.get('anniversary_employees')
        self.wish_type       = wish_type
        self.title           = 'Birthday'
        self.employees       = ','.join(self.birth_employees)
        if self.wish_type == '2':
            self.title = 'Work Anniversary'
            self.employees = ','.join(self.ann_employees)

    def run(self):
        self.send_generic_email()

    def send_generic_email(self):

        print("\n********** SENDING EMAIL ****************\n")
        print("SETTINGS ARE AS FOLLOWS:  \n")
        print(f"EMAIL_SERVER  = {self.GMAIL_SERVER}")
        print(f"OUTGOING_PORT = {self.GMAIL_PORT}\n")

        if not self.GMAIL_PORT or not self.GMAIL_SERVER or not self.GMAIL_SENDER or \
                not self.GMAIL_PASSWORD or not self.GMAIL_RECEIVER:
            print("*****  PLEASE ENTER YOUR SERVER AND PORT YOU'D LIKE TO USE DIFFERENT ONES.\n")

            input_server      = input('Please your mail server:      ')
            input_port        = input('Please your mail SMTP Port:      \n')
            email_address     = input('Enter the sender email:   ')
            email_password    = input('Enter the sender Password:   ')
            rcv_email_address = input('Enter the receiver Email:   ')

            if input_server:
                self.GMAIL_SERVER = input_server
            if input_port:
                self.GMAIL_PORT = int(input_port)
            if email_address:
                self.GMAIL_SENDER = email_address
            if email_password:
                self.GMAIL_PASSWORD = email_password
            if rcv_email_address:
                self.GMAIL_RECEIVER = rcv_email_address

            if not self.crontab and (not input_server or not input_port or not email_address or
                                     not email_password or not rcv_email_address):
                print("\nSENDER'S EMAIL & PASSWORD, AND RECEIVER EMAIL ARE REQUIRED!")
                print('EMAIL NOT SENT !!!\n')
                print('******** THANK YOU FOR USING OUR APP *********')
                time.sleep(1)
                return

        # Create a secure SSL context
        ssl_context = ssl.create_default_context()

        try:

            with smtplib.SMTP(self.GMAIL_SERVER, self.GMAIL_PORT) as wish_server:
                wish_server.ehlo()
                wish_server.starttls(context=ssl_context)
                wish_server.ehlo()

                wish_server.login(self.GMAIL_SENDER, self.GMAIL_PASSWORD)

                subject = f"{self.title} wishes."
                body = f"Happy {self.title} {self.employees}"
                msg = f"Subject: {subject}\n\n{body}"

                wish_server.sendmail(self.GMAIL_SENDER, self.GMAIL_RECEIVER, msg)
                print('EMAIL SENT **********\n')
                print('******** THANK YOU FOR USING OUR APP *********')
                time.sleep(1)

        except Exception as e:
            print('Trouble sending this email.')
            raise e



import pathlib
import time

import requests
from os                                         import path

from .core.employee                              import Employee
from .core.mail                                  import SendWishEmail
from .core.process                               import ProcessEmployee
from .db.data                                    import JSONData
from .operation                                  import Operation as ops


class GetInput:
    current_date = ops.get_today_date()

    def __init__(self, selection, **kwargs):
        self.selection   = selection
        self.crontab     = kwargs.get('crontab')
        self.emp_list    = []
        self.emp_na_list = []
    
    def compare_api_and_local_data(self):
        for employee in self.employees_local:
            if self.worker['id'] == employee['id']:
                str_date = ''
                lastNotification     = employee.get('lastNotification')
                lastBirthdayNotified = employee.get('lastBirthdayNotified')
                if lastNotification:
                    str_date = lastNotification
                elif lastBirthdayNotified:
                    str_date = lastBirthdayNotified

                if str_date:
                    last_date = ops.str_to_date(str_date)
                    if self.current_date.month == last_date.month and self.current_date.day == last_date.day:
                        return True
        return False

    def call_real_endpoint(self):

        employees     = []
        exc_employees = []
        emp_resp      = requests.get('https://interview-assessment-1.realmdigital.co.za/employees')
        exc_emp_resp  = requests.get('https://interview-assessment-1.realmdigital.co.za/do-not-send-birthday-wishes')

        if emp_resp.status_code == 200:
            employees = emp_resp.json()
        if exc_emp_resp.status_code == 200:
            exc_employees = exc_emp_resp.json()

        data = {
            'employees'     : employees,
            'exc_employees' : exc_employees
        }

        return data

    def get_input_redirect(self):

        birthday_names       = []
        anniversary_names    = []
        self.employees_local = []

        data_api      = self.call_real_endpoint()
        employees_api = data_api['employees']
        employees     = employees_api

        current_dir = pathlib.Path(__file__).parent
        json_file   = f'{current_dir}/db/employees.json'

        if path.isfile(json_file):
            data_local           = JSONData(None, None, None)
            self.employees_local = data_local.extract_employee_data(json_file)
            employees = self.employees_local

        # BIRTHDAY WISHES
        if self.selection == '1':
            self.wish_message_sender(employees, birthday_names)

        # ANNIVERSARY WISHES
        elif self.selection == '2':
            self.wish_message_sender(employees, anniversary_names)

        else:
            print('\t *** NO VALID SELECTION RECORDED ***')
            print('\t *** PROGRAM WILL EXIT NOW ***')
            time.sleep(2)

        return

    def wish_message_sender(self, employees, wish_names):

        for worker in employees:
            self.worker = worker

            if self.compare_api_and_local_data():
                continue

            employee = Employee(self.worker, employees)

            if self.selection == '1':
                is_valid = ProcessEmployee(employee).validate_birthdays_employees()
            else:
                is_valid = ProcessEmployee(employee).validate_anniversary_employees()

            if is_valid:
                # BirthdayMessage(employee)
                wish_names.append(employee.name)
                print(employee.name)
                self.worker['lastNotification'] = employee.get_current_date.strftime('%Y-%m-%d')

                if self.selection == '1':
                    self.worker['lastBirthdayNotified'] = employee.get_current_date.strftime('%Y-%m-%d')

                self.emp_list.append(self.worker)
            else:
                self.emp_na_list.append(self.worker)

        if self.emp_list:
            JSONData(self.emp_list, None, 'Y', self.emp_list, None).start()

        if self.emp_na_list:
            JSONData(None, self.emp_na_list, 'N', None, self.emp_na_list).start()

        if wish_names:
            SendWishEmail(self.selection, wish_names, crontab=self.crontab).start()
        else:
            if self.selection == '1':
                print('\n ****** NO EMPLOYEE HAS A BIRTHDAY TODAY *******')

            if self.selection == '2':
                print('\n ****** NO EMPLOYEE HAS A WORK-ANNIVERSARY TODAY *******')

        return self.selection



import calendar
from datetime                               import datetime


class Operation:

    @staticmethod
    def is_leap_year(year):
        return calendar.isleap(year)

    @staticmethod
    def get_today_date():
        return datetime.today().date()

    @staticmethod
    def str_to_date(str_date):
        if 'T' in str_date:
            return datetime.strptime(str_date.rsplit('T', 1)[0], "%Y-%m-%d").date()
        if '+' in str_date:
            return datetime.strptime(str_date.rsplit('+', 1)[0], "%Y-%m-%dT%H:%M:%S").date()
        elif ':' not in str_date:
            return datetime.strptime(str_date.rsplit('+', 1)[0], "%Y-%m-%d").date()
    


from setuptools                         import setup, find_packages


setup(
    name='realmz',
    version='0.0.16',
    license='MIT',
    author='Billy Katalayi',
    author_email='billysbn7@gmail.com',
    description="To send birthday wishes to employees",
    long_description='''\n\n
    Realmz\n\n
    
    This is a very small module built for a private project. Should you want to do more than the basis functionalities\n 
    supported by this package, then you are looking on the wrong place. A service component that will send birthday\n 
    wishes to employees. The service extracts a list of employees whose birthdays occur today using the Realm Digital\n 
    Employee API and create a generic message E.g. “Happy Birthday {name 1}, {name 2}” and send the message to an \n
    email address configured for the component. https://interview-assessment-1.realmdigital.co.za/\n\n\n
    
    Example:\n
        from realmz import main as realmz_main\n\n
        
        def test():\n
            realmz.start_wish()\n\n
            
        test()\n\n
                        
    You'd input the database full path in case you'd like to save data to a database instead of a json file.\n
    Please note that we currently only support sqlite.\n\n
    
    When prompted with options:
        1. Birthday
        2. Anniversary\n\n
        
    This package uses 'https://pypi.org/project/python-dotenv/' to handle .env variable for this project.\n\n
    
    This package uses 'https://pypi.org/project/schedule/' to schedule the app, when and how it would run.\n
    ''',
    packages=find_packages('src'),
    package_dir={'' : 'src'},
    url='https://github.com/Billykat7/wishes',
    keywords='employees birthday, anniversary automated wishes',
    project_urls = {
        "Bug Tracker": 'https://github.com/Billykat7/wishes/issues',
    },
    classifiers = [
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent"
    ],
    install_requires=[
        # 'celery',
        'python-dotenv',
        # 'redis',
        'requests',
        'schedule',
        # 'SQLAlchemy'
    ],
    include_package_data=True,
    python_requires = ">=3.6"
)



import schedule
import time
from .factory                                    import GetInput
from .settings                                   import Settings


SETTINGS = Settings(crontab='Y')
SETTINGS.get_settings()


def get_input(selection):
    GetInput(selection, crontab='Y').get_input_redirect()
    return

schedule.every().day.at("09:00").do(get_input, selection='1')
schedule.every().day.at("09:02").do(get_input, selection='2')

while True:
    schedule.run_pending()
    time.sleep(1)



from abc                            import ABC, abstractmethod


class ABCMessage(ABC):

    def process_email_msg(self, message):
        is_valid = self.validate_email_msg(message)
        if is_valid:
            self.send_email_notification()
        return is_valid

    def validate_email_msg(self, message):
        pass

    @classmethod
    @abstractmethod
    def send_email_notification(self):
        pass
